# Generated by roxygen2: do not edit by hand

export(aggregateBioVar)
export(countsByCell)
export(countsBySubject)
export(scSubjects)
export(subjectMetaData)
export(summarizedCounts)
importFrom(SingleCellExperiment,SingleCellExperiment)
importFrom(rlang,"!!")
importFrom(rlang,":=")
